@@ -3,13 +3,15 @@ import unittest2
 import doctest
 
 def get_suite():
-    # no tests under python 3 since we simply defer to the vanilla module.
     if sys.version_info >= (3, 0):
-        return unittest2.TestSuite()
+        start_module = 'unicodecsv.py3'
+    else:
+        start_module = 'unicodecsv.py2'
 
     loader = unittest2.TestLoader()
-    suite = loader.discover('unicodecsv.py2')
-    suite.addTest(doctest.DocTestSuite('unicodecsv.py2'))
+    suite = loader.discover(start_module)
+    suite.addTest(doctest.DocTestSuite(start_module))
+    suite.addTest(doctest.DocFileSuite('README.rst', optionflags=doctest.ELLIPSIS))
 
     return suite
 
@@ -20,4 +22,4 @@ def main():
         print(error)
 
 if __name__ == '__main__':
-    main()
\ No newline at end of file
+    main()
